// file      : xsd/cxx/xml/dom/parsing-header.hxx
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_CXX_XML_DOM_PARSING_HEADER_HXX
#define XSD_CXX_XML_DOM_PARSING_HEADER_HXX

namespace xsd
{
  namespace cxx
  {
    namespace xml
    {
      namespace dom
      {
        template <typename C>
        class parser;
      }
    }
  }
}

#endif // XSD_CXX_XML_DOM_PARSING_HEADER_HXX
